package com.clx.performance.enums;

import lombok.AllArgsConstructor;
import lombok.Getter;

import java.util.Arrays;
import java.util.Optional;

public class InternalMessageTemplateEnum {

    @Getter
    @AllArgsConstructor
    public enum TemplateType {
        GO_TO_SEND_TIMEOUT(PlatformType.CARRIER, MessageType.WARN, BusinessType.CHILD_WARN,BusinessTypeDetail.GO_TO_SEND_TIMEOUT),
        ;
        private final PlatformType platform;      //承运平台  司机APP
        private final MessageType messageType;   //预警、待办、通知、上报
        private final BusinessType businessType;   //业务大类   运单预警
        private final BusinessTypeDetail businessDetailType;  //业务详细分类   前往目的地超时等
    }



    @Getter
    @AllArgsConstructor
    public enum BusinessTypeDetail {


        ARRIVE_SEND_DELAY("GO_TO_SEND_TIMEOUT", "预警通知：到达货源地延误"),
        GO_TO_SEND_TIMEOUT("GO_TO_SEND_TIMEOUT", "预警通知：前往货源地超时"),
        ;

        private final String code;
        private final String title;


        public static Optional<BusinessTypeDetail> getByCode(String code) {
            return Arrays.stream(values()).filter(e -> e.code.equals(code)).findFirst();
        }
        public static String getTitle(String code) {
            if (code == null) {return null;}
            return getByCode(code).map(BusinessTypeDetail::getTitle).orElse(null);
        }
    }


    @Getter
    @AllArgsConstructor
    public enum BusinessType {
        CHILD_WARN("CHILD_WARN", "运单预警"),
        LINE_WARN("LINE_WARN", "线路预警"),
        WAIT_QUOTATION_ORDER("WAIT_QUOTATION_ORDER", "待报价订单"),
        WAIT_APPROVED_ITEM("WAIT_APPROVED_ITEM", "待审核事项"),
        WAIT_CREATE_ORDER_GOODS("WAIT_CREATE_ORDER_GOODS", "待创建货单"),
        ORDER_NOTICE("ORDER_NOTICE", "订单通知"),
        COMPLAINT_NOTICE("COMPLAINT_NOTICE", "投诉通知"),
        DRIVER_REPORT("DRIVER_REPORT", "司机上报"),
        INTERNAL_REPORT("INTERNAL_REPORT", "内部上报"),

        ;

        private final String code;
        private final String name;
        public static Optional<BusinessType> getByCode(String code) {
            return Arrays.stream(values()).filter(e -> e.code.equals(code)).findFirst();
        }

        public static String getName(String code) {
            if (code == null) {return null;}
            return getByCode(code).map(BusinessType::getName).orElse(null);
        }
    }

    @Getter
    @AllArgsConstructor
    public enum PlatformType {
        CARRIER("carrier-platform-npc", "承运平台"),
        DRIVER_APP("carrier-driver-app", "司机APP"),

        ;

        private final String code;
        private final String name;



        public static Optional<PlatformType> getByCode(String code) {
            return Arrays.stream(values()).filter(e -> e.code.equals(code)).findFirst();
        }

        public static String getName(String code) {
            if (code == null) {return null;}
            return getByCode(code).map(PlatformType::getName).orElse(null);
        }
    }




    //预警、待办、通知、上报
    @Getter
    @AllArgsConstructor
    public enum MessageType {
        WARN("WARN", "预警"),
        WAIT_DEAL("WAIT_DEAL", "待办"),
        NOTICE("NOTICE", "通知"),
        REPORT("REPORT", "上报"),

        ;

        private final String code;
        private final String name;



        public static Optional<MessageType> getByCode(String  code) {
            return Arrays.stream(values()).filter(e -> e.code.equals(code)).findFirst();
        }

        public static String getName(String  code) {
            if (code == null) {return null;}
            return getByCode(code).map(MessageType::getName).orElse(null);
        }
    }











}
